#!/bin/bash

# Define the actions when usage information is needed.
function usage {
	echo "Usage: $0 cp"
	echo "   or: $0 mycp bufsize"
	exit
}

# Define the actions when syntax errors occur.
function die {
	echo "The syntax of the command is incorrect."
	echo "Enter $0 --help for usage information."
	exit 1
}

# Display the usage if --help is requested.
if [ "$1" == "--help" ]; then usage; fi

# Execute `cp` if the user intended to trace `cp`.
if [ "$1" == "cp" ]; then
	if [ $# -ne 1 ]; then die; fi
	EXE=cp
	echo "Running cp..."

# Execute `mycp` if the user intended to trace `mycp`.
else
	if [ "$1" != "mycp" ]; then die; fi
	if [ $# -ne 2 ]; then die; fi

	EXE="./mycp $2"
	echo "Running mycp with buffer size $2..."
fi

# Remove the related files that might pre-exist.
rm -f trace.txt input.txt output.txt

# Loop 6 times for 6 different file sizes.
COUNT=0
SIZE=1
while [ $COUNT -lt 6 ]; do

	# Create files with sizes from 10 to 1000000 bytes.
	let COUNT=COUNT+1
	let SIZE=SIZE*10
	let ITER=SIZE/10
	while [ $ITER -gt 0 ]; do
		let ITER=ITER-1
		echo 012345678 >> input.txt
	done

	# Call `cp` or `mycp` and trace the function call.
	strace -o trace.txt $EXE input.txt output.txt

	# Display the number of `write` calls in trace output file.
	# Display the first occurrence of the `write` function call.
	echo "File size: $SIZE bytes"
	echo "  Occurrences of 'write':      `grep write -c trace.txt`"
	echo "  First occurrence of 'write': `grep write -m 1 trace.txt`"
	echo ""

	# Remove the generated files, input.txt and output.txt.
	rm -f input.txt output.txt

	# Comment out this line if you wish to view the trace.txt as well.
	rm -f trace.txt

done

# Terminate the script.
echo "Done."
exit

